@@ -534,10 +534,14 @@ SEXP Arith_mpfr_d(SEXP x, SEXP y, SEXP op)
     SET_MISMATCH;
     for(i=0; i < n; i++) {
 	double yi = yy[i % ny];
-	int y_is_int = (yi == trunc(yi) && LONG_MIN <= yi && yi <= LONG_MAX);
+	int y_is_int = (yi == (long)yi);
 
 	R_asMPFR(VECTOR_ELT(xD, i % nx), x_i);
 	if(y_is_int) { /* can use <mpfr> o <integer>  routines */
+#ifdef DEBUG_Rmpfr
+	    REprintf("y[i] (= %.20g is int in [L_MIN, L_MAX]=[%ld,%ld]: (long)* = %ld\n",
+		     LONG_MIN, LONG_MAX, yi, (long)yi);
+#endif
 	    switch(i_op) {
 	    case  1: /*  +  */ mpfr_add_si(x_i, x_i, (long)yi, MPFR_RNDN); break;
 	    case  2: /*  -  */ mpfr_sub_si(x_i, x_i, (long)yi, MPFR_RNDN); break;
@@ -629,7 +633,6 @@ SEXP Arith_d_mpfr(SEXP x, SEXP y, SEXP op)
     for(i=0; i < n; i++) {
 	double xi = xx[i % nx];
 	int x_is_int = (xi == trunc(xi) && LONG_MIN <= xi && xi <= LONG_MAX);
-
 	R_asMPFR(VECTOR_ELT(yD, i % ny), y_i);
 	if(x_is_int) { /* can use  <integer> o <mpfr>  routines */
 /* 	    REprintf("x[i] (= %g) is int: (long)* = %ld\n", xi, (long)xi); */
